/**
 * iPadImageCapture.js
 * Designed as a singleton module N.B. If state is held globally in this module, it will possibly
 * be shared amongst multiple concurrent invocations of this module's methods. If state is required,
 * then make this module instance based instead of a singleton.
 * 
 * Created By Ben Morrison on 2012-09-11
 * @copyright FIS Mobile 2014. All rights reserved.
 */

// The usual requires we need.
var ImageProcessing = require('/ui/common/ImageProcessing');
var CameraFactory = require('/ui/common/CameraFactory');

/**
 * Image capture and processing for iPad
 * @param args {
 *   infoText: info overlay text
 *   checkSideInfoText: check side info overlay text
 *   imageWidth: 
 *   imageCompression:
 *   imageColor:
 *   helperView:
 *   checkSide: 'front' or 'back'
 *   processingFn: Called just prior to processing the captured image
 *     Signature: processingFn()
 *   successFn: Called after procoessing the captured image
 *     Signature: successFn(args) where
 *       @param args {
 *         capturedImage: raw image captured by the camera
 *         thumbnailImage: thumbnail of the raw image
 *         depositImage: image which has been processed to meet the FI's requirements for deposit
 *       }
 *   cancelFn: Called when user cancelled capture
 *     Signature: cancelFn()
 *   errorFn: Called when some error has occured
 *     Signature: errorFn(errorMessage)
 */
exports.captureCheckImage = function(args) {
	Ti.API.error("capturedCheckImage");
	var overlay = null;
	var imageCaptureView = null;
	var imagePreviewBackground = null;
	var imagePreviewView = null;
	var that = this;
	
	var camera = CameraFactory.create({checkSide: args.checkSide});

	// Overlay that gets attached to the camera.
    overlay = Ti.UI.createView();
	overlay.addEventListener('touchstart', tickleTimer);

	// Image capture view
	imageCaptureView = createImageCaptureView({
		infoText: args.infoText,
		checkSideInfoText: args.checkSideInfoText,
		captureFn: function() {
			camera.takePicture();
		},
		cancelFn: function() {
			endCapture(args.cancelFn);
		}
	});
	overlay.add(imageCaptureView);

	// Image preview background
	imagePreviewBackground = createImagePreviewBackground();

	

	// Image preview view
	imagePreviewView = createImagePreviewView({
		acceptFn: function(previewedImage) {

			endCapture(args.processingFn);

			var helperView = Ti.UI.createView(); 

			ImageProcessing.processCheckImage(
				previewedImage, args.imageWidth, args.imageCompression, args.imageColor,
				function(processedImage, thumbnailImage) { // successful process check image
					args.successFn && args.successFn.call(that, processedImage, thumbnailImage, previewedImage);
				},
				function() { // error process check image
					args.errorFn && args.errorFn.apply(that, arguments);
				},
				helperView);
		},
		retakeFn: function() {
			switchToImageCapture();
		}
	});

	// Set view for current orientation. 
	device_onOrientationChange({orientation: Ti.UI.orientation});

	// Listen for rotation 
	// Comment out so the overlay doesnt move. 
	// Ti.Gesture.addEventListener('orientationchange', device_onOrientationChange);

	// Show the camera
	camera.showCamera({

		overlay: overlay,

		successFn: function(event) {
			var capturedImage = ImageProcessing.rotateImage(event.media);
			switchToImagePreview(capturedImage);
		},

		errorFn: function(error) {
			// Do nothing for this test example
		}
	});

	//--------------------------------------------------------------------------------------------
	// Support functions
	//--------------------------------------------------------------------------------------------

	/**
	 * Image capture view. Displays the camera so the user can take a picture, exit the camera,
	 * or display help.
	 * @param args {
	 *   infoText:
	 *   checkSideInfoText:
	 *   captureFn:
	 *   cancelFn:
	 * }
	 */
	function createImageCaptureView(args) {
		var dims = getDims();

		var view = Ti.UI.createView({
			width: dims.M,
			height: dims.m,
			zIndex: 1
		});

		var toolbar = Ti.UI.createView({
			width: 64,
  			height: Ti.UI.FILL,
  			top: 0,
  			bottom: 0,
  			right: 0,
  			backgroundGradient: {
	        	colors: ["#303030", "#202020", "#181818"],
	        	startPoint: {x: 0.0, y: 0.0},
	        	endPoint: {x: 100, y: 0}
        	}
  		});

		view.add(toolbar);

		var cameraBtn = Ti.UI.createButton({
			backgroundImage: "/images/rdc/camera-up.png",
    		backgroundSelectedImage: "/images/rdc/camera-down.png",
    		backgroundDisabledImage: "/images/rdc/camera-down.png",
  			width: 56,
    		height: 56
    	});
		toolbar.add(cameraBtn);
		cameraBtn.addEventListener('click', function() {
			args.captureFn();
		});

		var cancelBtn = Ti.UI.createButton({
			width: 28,
			height: 28,
			backgroundImage: "/images/rdc/close.png",
			backgroundSelectedImage: "/images/rdc/close-down.png",
			backgroundDisabledImage: "/images/rdc/close-down.png",
			center : {
				y : "65%"
			}
		});

		toolbar.add(cancelBtn);
		cancelBtn.addEventListener('click', function() {
			args.cancelFn();
		});

		var alignImg = Ti.UI.createImageView({
			center : {
    			x: "47.2%"
  			},
    		image: "/images/rdc/camera-align.png",
		});
		view.add(alignImg);

		var infoView = Ti.UI.createView({
			top: 0,
  			right: 64,
			left: 0,
			height: Ti.UI.SIZE,
			backgroundColor: "black",
			opacity: 0.5
		});
		view.add(infoView);

		var infoLabel = Ti.UI.createLabel({
			top: 10,
			right: 10,
			left: 10,
			color: "white",
			textAlign: "center",
			font: {fontSize: "20dp", font: "Helvetica Neue"},
			text: args.infoText,
		});
		infoView.add(infoLabel);

		var checkSideInfoView = Ti.UI.createView({
			right: 64,
			bottom: 0,
			left: 0,
			height: Ti.UI.SIZE,
			backgroundColor: "black",
			opacity: 0.5
		});
		view.add(checkSideInfoView);

		var checkSideInfoLabel = Ti.UI.createLabel({
			top: 10,
			right: 10,
			left: 10,
			color: "white",
			font: {fontSize: "55dp", font: "Helvetica Neue"},
			textAlign: "center",
			text: args.checkSideInfoText,
		});
		checkSideInfoView.add(checkSideInfoLabel);

		return view;
	}

	/**
	 * Return a view large enough so that when the device rotates we don't see the image capture
	 * view beneath the image preview view.
	 */
	function createImagePreviewBackground() {
		var dims = getDims();
		return Ti.UI.createView({
				backgroundColor : "black",
				width: dims.M,
				height: dims.M,
				zIndex: 2
		});
	}

	/**
	 * Image preview view. Displays the captured image so the user can preview it. They can accept
	 * or retake the image.
	 * @param args {
	 *   acceptFn:
	 *   retakeFn:
	 * }
	 */
	function createImagePreviewView(args) {

		var dims = getDims();
		var view = Ti.UI.createView({
				backgroundColor : "black",
				width: dims.M,
				height: dims.m,
				zIndex: 3,
			});

		// Attach a mutator to the actual preview view object so that we don't have to expose the
		// image view just to set the image within.
		view.putImage = function(image) {
			imageView.setImage(image);
			imageView.visible = true;
		}

		// Attach a cleanup method
		view.cleanup = function() {
			view.remove(imageView);
			delete imageView.image;
		}

		var imageView = Ti.UI.createImageView({
				backgroundColor: "black",
				left: 0,
				width: dims.M,
				height: dims.m,
				visible: false
			});
		view.add(imageView);

		var toolbar = Ti.UI.createView({
			width: 64,
  			height: Ti.UI.FILL,
  			top: 0,
  			bottom: 0,
  			right: 0,
  			backgroundGradient: {
		    	colors: ["#303030", "#202020", "#181818"],
		    	startPoint: {x: 0.0, y: 0.0},
		    	endPoint: {x: 100, y: 0}
        	}
		});
		view.add(toolbar);

		var acceptBtn = Ti.UI.createButton({
			backgroundImage: "/images/rdc/accept-btn-up.png",
			backgroundSelectedImage: "/images/rdc/accept-btn-down.png",
			backgroundDisabledImage: "/images/rdc/accept-btn-down.png",
			width: 56,
			height: 56
		});
		toolbar.add(acceptBtn);
		acceptBtn.addEventListener('click', function() {
			// N.B., we are sending back the actual image that's still attached to our image
			// view. Hopefully modifications to the image don't cause the view to barf and the
			// app to die. Otherwise we would need a copy method or need to destroy the view
			// prior to processing.
			var previewedImage = imageView.image;
			args.acceptFn(previewedImage);
		});

		var retakeBtn = Ti.UI.createButton({
			width: 38,
			height: 38,
			backgroundImage: "/images/rdc/retake-btn-up.png",
			backgroundSelectedImage: "/images/rdc/retake-btn-down.png",
			backgroundDisabledImage: "/images/rdc/retake-btn-down.png",
			center : {
				y : "65%"
			}
		});
		toolbar.add(retakeBtn);
		retakeBtn.addEventListener('click', function() {
			imageView.visible = false;
			args.retakeFn();
		});

		return view;
	}

	function device_onOrientationChange(e) {
		Ti.API.error('orientation change: ' + JSON.stringify(e));
		switch (e.orientation) {
		case Ti.UI.LANDSCAPE_LEFT:
			displayAtAngleOf(180); // upside down!
			break;
		case Ti.UI.LANDSCAPE_RIGHT:
			displayAtAngleOf(0); // right side up
			break;
		case Ti.UI.PORTRAIT:
			displayAtAngleOf(90); // text flows top to bottom
			break;
		case Ti.UI.UPSIDE_PORTRAIT:
			displayAtAngleOf(270); // text flow bottom to top
			break;
		}
	}

	/**
	 * Display the views at the given angle
	 * @param angle 0, views are upright where the device's home button is on the right. It goes
	 * clockwise from there.
	 */
	function displayAtAngleOf(angle) {
		Ti.API.error('displaying at angle of: ' + angle);
		imageCaptureView.transform = Ti.UI.create2DMatrix({rotate: angle});
		imagePreviewView.transform = Ti.UI.create2DMatrix({rotate: angle});
	}

	function switchToImagePreview(image) {
		overlay.add(imagePreviewBackground);
		overlay.add(imagePreviewView);
		imagePreviewView.putImage(image);
	}

	function switchToImageCapture() {
		overlay.remove(imagePreviewView);
		overlay.remove(imagePreviewBackground);
	}

	function endCapture(exitFn) {
		overlay.removeEventListener('touchstart', tickleTimer);
		// Ti.Gesture.removeEventListener('orientationchange', device_onOrientationChange);
		camera.hideCamera();

		overlay.remove(imageCaptureView);	
		overlay.remove(imagePreviewBackground);
		overlay.remove(imagePreviewView);
			
		imagePreviewView.cleanup();

		exitFn && exitFn.call(that);
	}

	function tickleTimer() {
		// DOes nothing for sample app
	}


	/**
	 * @return Absolute dimensions. The largest dimension (M) and the smallest dimension (m).
	 */
	function getDims() {
		return {
			M: Math.max(Ti.Platform.displayCaps.platformHeight,
						Ti.Platform.displayCaps.platformWidth),
			m: Math.min(Ti.Platform.displayCaps.platformHeight,
						Ti.Platform.displayCaps.platformWidth)
		}
	}
}

// Function to allow cleanup of certain variables outside of the iPadImageCapture page. 
exports.cleanUp = function(){
	imageCaptureView = null;
	overlay = null;
	imagePreviewBackground = null;
	imagePreviewView = null;
};